const db = getDb("questions.sqlite");
db.table(
  "questions",
  `
    create table questions (
      qmid integer not null,
      type integer not null,
      text text not null,
      primary key (qmid)
    );
  `,
);

export enum QuestionType {
  normal = 0,
  reject = 1,
  unused = 2,
  pending = 3,
}

export class Question {
  /**
   * questions are identified by a 'minute identifier'
   * which is time / 60000 (minutes since epoch)
   */
  qmid!: number;
  type!: QuestionType;
  text!: string;

  // -- instance ops --

  /** "https://paperclover.net/q+a/${id}" - #YYMMDDHHMM */
  get id(): string {
    return formatQuestionId(this.date);
  }
  get date(): Date {
    return new Date(Number(this.qmid) * 1000 * 60);
  }

  // -- static ops --

  /** Create a new Question, rounding the date to the nearest available slot */
  static create(type: QuestionType, text: string, date?: Date) {
    const qmidWanted = Math.floor((date?.getTime() ?? Date.now()) / 1000 / 60);
    const { qmid } = createQuery.getNonNull({
      type,
      text,
      qmid: qmidWanted,
    });

    return new Date(qmid * 1000 * 60);
  }
  static getAll() {
    return getAllQuery.iter();
  }
  static getByDate(timestamp: Date) {
    // Older questions are inserted with second precision,
    // but that data should always be floored.
    const qmid = Math.floor(timestamp.getTime() / 1000 / 60);
    return getByDateQuery.get(qmid);
  }
  static deleteByQmid(qmid: number) {
    return deleteByQmidQuery.run(qmid);
  }
  static rejectByQmid(qmid: number) {
    return rejectByQmidQuery.run(qmid);
  }
  static updateByQmid(qmid: number, text: string, type: QuestionType) {
    return updateByQmidQuery.run({ text, type, qmid });
  }
}

// Create a new question with a unique QMID
const createQuery = db.prepare<
  [{ type: QuestionType; text: string; qmid: number }],
  { qmid: number }
>(/* SQL */ `
  WITH RECURSIVE
  timestamp(value) AS (
    SELECT $qmid
    UNION ALL
    SELECT value + 1
    FROM timestamp
    WHERE EXISTS (SELECT 1 FROM questions WHERE qmid = value)
  )
  INSERT INTO questions (type, text, qmid)
  VALUES ($type, $text, (
    SELECT value FROM timestamp 
    WHERE NOT EXISTS (SELECT 1 FROM questions WHERE qmid = value)
    LIMIT 1
  ))
  RETURNING qmid
`);

const getAllQuery = db.prepare(/* SQL */ `
  SELECT * FROM questions WHERE type = ${QuestionType.normal} ORDER BY qmid DESC
`).as(Question);

const getByDateQuery = db.prepare<[qmid: number]>(`
  SELECT * FROM questions WHERE qmid = ? LIMIT 1
`).as(Question);

const deleteByQmidQuery = db.prepare<[qmid: number]>(`
  DELETE FROM questions WHERE qmid = ?
`);

const rejectByQmidQuery = db.prepare<[qmid: number]>(`
  UPDATE questions SET type = ${QuestionType.reject} WHERE qmid = ?
`);

const updateByQmidQuery = db.prepare<
  [{ qmid: number; text: string; type: QuestionType }]
>(`
  UPDATE questions SET text = $text, type = $type WHERE qmid = $qmid
`);

import { formatQuestionId } from "../format.ts";
import { getDb } from "#sitegen/sqlite";
